package com.winhc.repal.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.winhc.common.base.BeanResponse;
import com.winhc.common.enums.CodeMsg;
import com.winhc.common.exception.CommonException;
import com.winhc.repal.cloud.WinCoinService;
import com.winhc.repal.cloud.dto.WinCoinOrderFormDTO;
import com.winhc.repal.cloud.vo.WinCoinOrderFormVO;
import com.winhc.repal.entity.*;
import com.winhc.repal.enums.*;
import com.winhc.repal.model.bo.UserContextBO;
import com.winhc.repal.model.dto.*;
import com.winhc.repal.model.vo.RepalLinkVO;
import com.winhc.repal.model.vo.RepalSmsTemplateVO;
import com.winhc.repal.repository.RepalDynamicMapper;
import com.winhc.repal.repository.RepalVipMapper;
import com.winhc.repal.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.winhc.repal.util.UserContextUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * @description RepalDynamic 接口实现类
 * @author Generator
 * @date 2022-05-06
 */
@Service
public class RepalDynamicServiceImpl extends ServiceImpl<RepalDynamicMapper, RepalDynamic> implements RepalDynamicService {
    @Autowired
    private RepalBillService repalBillService;

    @Autowired
    private RepalRolePermissionService repalRolePermissionService;

    @Autowired
    private MessageNoticeService messageNoticeService;

    @Autowired
    private RepalGroupService repalGroupService;

    @Autowired
    private WinCoinService winCoinService;

    @Autowired
    private RepalCustomerService repalCustomerService;

    @Autowired
    private RepalVipService repalVipService;

    @Autowired
    private RepalVipMapper repalVipMapper;

    @Override
    public Boolean addDynamic(DynamicContentDTO dto) {
        UserContextBO userContextBO = UserContextUtil.getUser();
        RepalDynamic dynamic = new RepalDynamic();
        dynamic.setRepalBillId(dto.getRepalBillId());
        dynamic.setDynamicDesc(dto.getDynamicDesc());
        dynamic.setDynamicType(RepalDynamicTypeEnum.其他.getCode());
        dynamic.setMemberId(userContextBO.getUserId());
        save(dynamic);
        return Boolean.TRUE;
    }

    @Override
    public List<RepalSmsTemplateVO> getSmsTemplate(Long repalBillId) {
        RepalBill repalBill = repalBillService.getById(repalBillId);
        if(ObjectUtil.isNull(repalBill)){
            throw new CommonException(CodeMsg.VALIDATE_PARAMETER,"账款不存在");
        }
        RepalGroup repalGroup = repalGroupService.getById(repalBill.getRepalGroupId());
        if(ObjectUtil.isNull(repalGroup)){
            throw new CommonException(CodeMsg.VALIDATE_PARAMETER,"组织不存在");
        }
        //目前只有一个，估计也只有一个
        RepalSmsTemplateVO result = new RepalSmsTemplateVO();
        DecimalFormat df = new DecimalFormat("#,###.00");
        long dayBetween = Math.abs(LocalDate.now().toEpochDay() - repalBill.getEndDate().toEpochDay());
        String overdueStr = RepalBillStatusEnum.PROGRESSING.getCode().equals(repalBill.getRepalBillStatus()) ? "还有"+dayBetween+"天即将逾期":"已逾期"+dayBetween+"天";
        String content = StrUtil.format(RepalMsgTemplateEnum.短信模板一.getContent(),
                "XXX",
                repalGroup.getGroupName(),
                df.format(repalBill.getTotalMoney()),
                df.format(repalBill.getReceivable()),
                repalBill.getEndDate().format(DateTimeFormatter.ISO_LOCAL_DATE),
                overdueStr);
        result.setSmsCode(RepalMsgTemplateEnum.短信模板一.getCode());
        result.setContent(content);
        return Collections.singletonList(result);
    }

    @Override
    @Transactional
    public Boolean urgeBySms(RepalSmsDTO dto) {
        //校验权限
        UserContextBO userContextBO = UserContextUtil.getUser();
        boolean allFlag = repalRolePermissionService.checkUserPermission(userContextBO.getUserId(),PermissionEnum.URGE_BILL.getCode());
        if(!allFlag){
            throw new CommonException(CodeMsg.FORBIDDEN,"暂无此权限，可联系超级管理员在我的-成员管理页面开通");
        }
        if(CollUtil.isEmpty(dto.getLinks())){
            throw new CommonException(CodeMsg.FAILED,"请至少选择一个联系人");
        }
        if(ObjectUtil.isNull(dto.getSmsCode())){
            throw new CommonException(CodeMsg.FAILED,"请选择发送模板");
        }
        RepalBill repalBill = repalBillService.getById(dto.getRepalBillId());
        List<RepalSmsTemplateVO> templates = getSmsTemplate(dto.getRepalBillId());
        if(ObjectUtil.isNull(repalBill) || CollUtil.isEmpty(templates)){
            throw new CommonException(CodeMsg.FAILED,"发送失败");
        }
        //扣减vip次数
        RepalVip repalVip = repalVipService.getOne(Wrappers.lambdaQuery(RepalVip.class).eq(RepalVip::getRepalGroupId,repalBill.getRepalGroupId()),false);
        if(ObjectUtil.isNull(repalVip) || RepalVipSignEnum.无特权.getCode().equals(repalVip.getRepalVipSign())){
            throw new CommonException(CodeMsg.FAILED,"请先开通特权");
        }
        int count = repalVipMapper.updateSomeCount(repalVip.getId(),RepalDynamicTypeEnum.短信.getCode());
        if(count < 1){
            throw new CommonException(CodeMsg.FAILED,"可用短信次数不足");
        }
        for(RepalSmsTemplateVO template:templates){
            if(template.getSmsCode().equals(dto.getSmsCode())){
                Map<String,Object> paraMap = new HashMap<>();
                paraMap.put("content",template.getContent());
                paraMap.put("company",repalBill.getCustomerName());
                paraMap.put("links",dto.getLinks());
                paraMap.put("source","REPAL");
                List<String> names = new ArrayList<>();
                for(RepalLinkVO link: dto.getLinks()){
                    if(StrUtil.isNotBlank(link.getMobileNo())) {
                        names.add(link.getName());
                    }
                }
                WinCoinOrderFormDTO winCoinOrderFormDTO = new WinCoinOrderFormDTO();
                winCoinOrderFormDTO.setUserId(userContextBO.getUserId());
                winCoinOrderFormDTO.setProductCode("sms_repal");
                winCoinOrderFormDTO.setPayMode("1");
                winCoinOrderFormDTO.setTransAmt(BigDecimal.ZERO);
                winCoinOrderFormDTO.setGoodsJson(JSON.toJSONString(paraMap));
                BeanResponse<WinCoinOrderFormVO> rsp = winCoinService.createOrder(JSON.toJSONString(userContextBO),userContextBO.getAppVersion(),userContextBO.getAppPackageName(),winCoinOrderFormDTO);
                if(!rsp.getIsSuccess().equals("T")){
                    throw new CommonException(CodeMsg.FAILED,rsp.getErrorMsg());
                }
                //记录催款动态
                String dynamicDesc = "接收人："+String.join("、",names)+"\n\r";
                dynamicDesc+="信息内容："+template.getContent();
                RepalDynamic dynamic = new RepalDynamic();
                dynamic.setRepalBillId(dto.getRepalBillId());
                dynamic.setDynamicDesc(dynamicDesc);
                dynamic.setDynamicType(RepalDynamicTypeEnum.短信.getCode());
                dynamic.setMemberId(userContextBO.getUserId());
                dynamic.setLinkMan(JSON.toJSONString(dto.getLinks()));
                dynamic.setOrderId(rsp.getBody().getOrderId());
                save(dynamic);
            }
        }
        return Boolean.TRUE;
    }

    @Override
    @Transactional
    public Boolean urgeByLawyerLetter(RepalLetterDTO dto) {
        //校验权限
        UserContextBO userContextBO = UserContextUtil.getUser();
        boolean allFlag = repalRolePermissionService.checkUserPermission(userContextBO.getUserId(),PermissionEnum.URGE_BILL.getCode());
        if(!allFlag){
            throw new CommonException(CodeMsg.FORBIDDEN,"暂无此权限，可联系超级管理员在我的-成员管理页面开通");
        }
        RepalBill repalBill = repalBillService.getById(dto.getRepalBillId());
        if(ObjectUtil.isNull(repalBill)){
            throw new CommonException(CodeMsg.FAILED,"发送失败,账款不存在");
        }
        //扣减vip次数
        RepalVip repalVip = repalVipService.getOne(Wrappers.lambdaQuery(RepalVip.class).eq(RepalVip::getRepalGroupId,repalBill.getRepalGroupId()),false);
        if(ObjectUtil.isNull(repalVip) || RepalVipSignEnum.无特权.getCode().equals(repalVip.getRepalVipSign())){
            throw new CommonException(CodeMsg.FAILED,"请先开通特权");
        }
        int count = repalVipMapper.updateSomeCount(repalVip.getId(),RepalDynamicTypeEnum.律师函.getCode());
        if(count < 1){
            throw new CommonException(CodeMsg.FAILED,"可用催款函次数不足");
        }
        Map<String,Object> paraMap = new HashMap<>();
        paraMap.put("company",repalBill.getCustomerName());
        paraMap.put("links",dto.getLinks());
        paraMap.put("source","REPAL");
        paraMap.put("productCode","303_1");
        WinCoinOrderFormDTO winCoinOrderFormDTO = new WinCoinOrderFormDTO();
        winCoinOrderFormDTO.setUserId(userContextBO.getUserId());
        winCoinOrderFormDTO.setProductCode("303");
        winCoinOrderFormDTO.setPayMode("1");
        winCoinOrderFormDTO.setTransAmt(BigDecimal.ZERO);
        winCoinOrderFormDTO.setGoodsJson(JSON.toJSONString(paraMap));
        BeanResponse<WinCoinOrderFormVO> rsp = winCoinService.createOrder(JSON.toJSONString(userContextBO),userContextBO.getAppVersion(),userContextBO.getAppPackageName(),winCoinOrderFormDTO);
        if(!rsp.getIsSuccess().equals("T")){
            throw new CommonException(CodeMsg.FAILED,rsp.getErrorMsg());
        }

        RepalDynamic dynamic = new RepalDynamic();
        dynamic.setRepalBillId(dto.getRepalBillId());
        dynamic.setDynamicType(RepalDynamicTypeEnum.律师函.getCode());
        dynamic.setMemberId(userContextBO.getUserId());
        dynamic.setLinkMan(JSON.toJSONString(dto.getLinks()));
        dynamic.setOrderId(rsp.getBody().getOrderId());
        save(dynamic);
        return Boolean.TRUE;
    }

    @Override
    @Transactional
    public Boolean urgeByConsult(RepalConsultDTO dto) {
        //校验权限
        UserContextBO userContextBO = UserContextUtil.getUser();
        boolean allFlag = repalRolePermissionService.checkUserPermission(userContextBO.getUserId(),PermissionEnum.URGE_BILL.getCode());
        if(!allFlag){
            throw new CommonException(CodeMsg.FORBIDDEN,"暂无此权限，可联系超级管理员在我的-成员管理页面开通");
        }
        RepalBill repalBill = repalBillService.getById(dto.getRepalBillId());
        if(ObjectUtil.isNull(repalBill)){
            throw new CommonException(CodeMsg.FAILED,"参数有误");
        }
        //扣减vip次数
        RepalVip repalVip = repalVipService.getOne(Wrappers.lambdaQuery(RepalVip.class).eq(RepalVip::getRepalGroupId,repalBill.getRepalGroupId()),false);
        if(ObjectUtil.isNull(repalVip) || RepalVipSignEnum.无特权.getCode().equals(repalVip.getRepalVipSign())){
            throw new CommonException(CodeMsg.FAILED,"请先开通特权");
        }
        int count = repalVipMapper.updateSomeCount(repalVip.getId(),RepalDynamicTypeEnum.咨询.getCode());
        if(count < 1){
            throw new CommonException(CodeMsg.FAILED,"咨询律师次数不足");
        }
        Map<String,Object> paraMap = new HashMap<>();
        paraMap.put("province",dto.getProvince());
        paraMap.put("city",dto.getCity());
        paraMap.put("serviceType",dto.getServiceType());
        paraMap.put("lawyerServiceSubType",102);
        paraMap.put("lawyerServiceType",1);
        paraMap.put("serviceDesc",dto.getServiceDesc());
        paraMap.put("otherArea",dto.getOtherArea());
        paraMap.put("source","REPAL");
        WinCoinOrderFormDTO winCoinOrderFormDTO = new WinCoinOrderFormDTO();
        winCoinOrderFormDTO.setUserId(userContextBO.getUserId());
        winCoinOrderFormDTO.setProductCode("phone_consult");
        winCoinOrderFormDTO.setPayMode("1");
        winCoinOrderFormDTO.setTransAmt(BigDecimal.ZERO);
        winCoinOrderFormDTO.setGoodsJson(JSON.toJSONString(paraMap));
        BeanResponse<WinCoinOrderFormVO> rsp = winCoinService.createOrder(JSON.toJSONString(userContextBO),userContextBO.getAppVersion(),userContextBO.getAppPackageName(),winCoinOrderFormDTO);
        if(!rsp.getIsSuccess().equals("T")){
            throw new CommonException(CodeMsg.FAILED,rsp.getErrorMsg());
        }

        RepalDynamic dynamic = new RepalDynamic();
        dynamic.setRepalBillId(dto.getRepalBillId());
        dynamic.setDynamicType(RepalDynamicTypeEnum.咨询.getCode());
        dynamic.setMemberId(userContextBO.getUserId());
        dynamic.setOrderId(rsp.getBody().getOrderId());
        save(dynamic);
        return Boolean.TRUE;
    }

    @Override
    @Transactional
    public Boolean urgeByLawsuit(RepalLawDTO dto) {
        //校验权限
        UserContextBO userContextBO = UserContextUtil.getUser();
        boolean allFlag = repalRolePermissionService.checkUserPermission(userContextBO.getUserId(),PermissionEnum.URGE_BILL.getCode());
        if(!allFlag){
            throw new CommonException(CodeMsg.FORBIDDEN,"暂无此权限，可联系超级管理员在我的-成员管理页面开通");
        }
        RepalBill repalBill = repalBillService.getById(dto.getRepalBillId());
        RepalCustomer repalCustomer = repalCustomerService.getById(repalBill.getCustomerId());
        if(ObjectUtil.isNull(repalBill) || ObjectUtil.isNull(repalCustomer)){
            throw new CommonException(CodeMsg.FAILED,"参数有误");
        }
        RepalVip repalVip = repalVipService.getOne(Wrappers.lambdaQuery(RepalVip.class).eq(RepalVip::getRepalGroupId,repalBill.getRepalGroupId()),false);
        if(ObjectUtil.isNull(repalVip) || RepalVipSignEnum.无特权.getCode().equals(repalVip.getRepalVipSign())){
            throw new CommonException(CodeMsg.FAILED,"请先开通特权");
        }
        int count = repalVipMapper.updateSomeCount(repalVip.getId(),RepalDynamicTypeEnum.诉讼.getCode());
        if(count < 1){
            throw new CommonException(CodeMsg.FAILED,"诉讼追款次数不足");
        }
        String productCode = "205";
        if(RepalBillStageEnum.NON_PROSECUTE.getCode().equals(dto.getCaseStage())){
            productCode = "203";
        }
        Map<String,Object> defendantMap = new HashMap<>();
        defendantMap.put("defendantName",repalCustomer.getCustName());
        defendantMap.put("defendantType",1);
        if(StrUtil.isNotBlank(repalCustomer.getCompanyId())){
            defendantMap.put("companyId",repalCustomer.getCompanyId());
            defendantMap.put("defendantType",2);
        }

        Map<String,Object> paraMap = new HashMap<>();
        paraMap.put("province",dto.getProvince());
        paraMap.put("city",dto.getCity());
        paraMap.put("serviceType",dto.getServiceType());
        paraMap.put("lawyerServiceSubType",productCode);
        paraMap.put("lawyerServiceType",2);
        paraMap.put("caseAmt",dto.getCaseAmt());
        paraMap.put("caseStage",dto.getCaseStage());
        paraMap.put("serviceDesc",dto.getServiceDesc());
        paraMap.put("defendantList", Collections.singletonList(defendantMap));
        paraMap.put("source","REPAL");
        WinCoinOrderFormDTO winCoinOrderFormDTO = new WinCoinOrderFormDTO();
        winCoinOrderFormDTO.setUserId(userContextBO.getUserId());
        winCoinOrderFormDTO.setProductCode(productCode);
        winCoinOrderFormDTO.setPayMode("1");
        winCoinOrderFormDTO.setTransAmt(BigDecimal.ZERO);
        winCoinOrderFormDTO.setGoodsJson(JSON.toJSONString(paraMap));
        BeanResponse<WinCoinOrderFormVO> rsp = winCoinService.createOrder(JSON.toJSONString(userContextBO),userContextBO.getAppVersion(),userContextBO.getAppPackageName(),winCoinOrderFormDTO);
        if(!rsp.getIsSuccess().equals("T")){
            throw new CommonException(CodeMsg.FAILED,rsp.getErrorMsg());
        }

        RepalDynamic dynamic = new RepalDynamic();
        dynamic.setRepalBillId(dto.getRepalBillId());
        dynamic.setDynamicType(RepalDynamicTypeEnum.诉讼.getCode());
        dynamic.setMemberId(userContextBO.getUserId());
        dynamic.setOrderId(rsp.getBody().getOrderId());
        save(dynamic);
        return Boolean.TRUE;
    }
}
